//***************************
//* BMP2TXT.C 1.3 ws 2/2017 *
//***************************

#include <stdio.h>
#include <string.h>

long filesize(char *fname);
int byterev(int wert);
void waitkey(void);

int main(int argc, char **argv)
{
	FILE *in;
	FILE *out;
	unsigned int x, y, z, blk, ch, ck, i, j;
    unsigned long fs;
    char outfile[20];
    char outline[100];
    char *filename, *name, *ext;
    unsigned char bmphead[62];
    unsigned char bmpdata[4096];
    unsigned char font[256][16];
    unsigned char newtab[4096];
	
    filename = strupr(argv[1]);

  	printf("BMP2TXT 1.3 ws 2/2017\n\n");

	if(argc < 2) {
    	printf("No File specified !!!\n\n");
        waitkey();
        return -1;
    }

    if(!strstr(filename,".BMP")) {  
    	printf("File has no .BMP Extension !!!\n\n");
        waitkey();
        return -2;
    }

	in = fopen(filename, "rb");
	if(!in) {
    	printf("File not found !!!\n\n");
        waitkey();
        return -3;
    }

    fs = filesize(filename);
    if((fs != 2110) && (fs != 3134) && (fs != 4158)) {
    	printf("ERROR! Filesize must be:\n\n 2.110 Bytes (128x128x2 BMP) or\n 3.134 Bytes (128x192x2 BMP) or\n 4.158 Bytes (128x256x2 BMP).\n\n");
        waitkey();
        return -4;
    }      
    
    name = strtok(filename, ".");
    ext = strtok(NULL, ".");
    strcpy(outfile,name);
    strcat(outfile,".TXT");

    if(fs == 2110) {
        blk = 2;      
    }      
    else if(fs == 3134) {
        blk = 3;      
    }      
    else if(fs == 4158) {
        blk = 4;      
    }      

	for(x = 0 ; x<62 ; x++) {
		bmphead[x] = fgetc(in);
    }

    if((int)bmphead[0] != 66  || (int)bmphead[1] != 77) {
    	printf("File has no BMP-Header !!!\n\n");
        waitkey();
        return -5;
    }

    if((int)bmphead[18] != 128) {
    	printf("BMP must have 128 Pixel Width !!!\n\n");
        waitkey();
        return -6;
    }

	for(x = 0 ; x<(blk*64) ; x++) {
        i = (blk * 1024) - ((x+1) * 16);
    	for(y = 0 ; y<16 ; y++) {
			bmpdata[i] = (char)byterev((int)fgetc(in));
            i++;
        }
    }
    fclose(in);

    i = 0;
	for(x = 0 ; x<16 ; x++) {
    	for(y = 0 ; y<16 ; y++) {
            ch = y + i;
            ck = y + (x * 16);
        	for(z = 0 ; z<(blk*4) ; z++) {
                font[ck][z] = bmpdata[ch];
                ch += 16;
            }
        }
        i += blk * 64;
    }

	for(x = 0 ; x<256 ; x++) {
        ch = x * 4;
		for(y=0 ; y<blk ; y++) {
            ck = ch; 
     		for(z=0 ; z<4 ; z++) {
                newtab[ck] = font[x][(y*4)+z];  
                ck++; 
            }
            ch += 1024;
        }
    }

    ch = 0;
	out = fopen(outfile, "wb");

    fprintf(out,"%s", "DAT\r\n\r\n");  
    fprintf(out,"%s", "font  long\r\n\r\n");  
    fprintf(out,"%s%i\r\n\r\n", "'     Font 8x", (blk*4));  

	for(x = 0 ; x<blk ; x++) {

        strcpy(outline,"'     ");
        if(x==0) {
            strcat(outline,"1.");
        }
        else if(x==1) {
            strcat(outline,"2.");
        }             
        else if(x==2) {
            strcat(outline,"3.");
        }             
        else if(x==3) {
            strcat(outline,"4.");
        }             
        strcat(outline," 4 Scanlines of Char ");

        strcat(outline,"00h - FFh");

        fprintf(out,"%s\r\n\r\n", outline);  
        for(i = 0 ; i<32 ; i++) {
            fprintf(out,"%s", "long  $");  
            for(j = 0 ; j<8 ; j++) {
                fprintf(out,"%02X", (int)newtab[ch+3]);  
                fprintf(out,"%02X", (int)newtab[ch+2]);  
                fprintf(out,"%02X", (int)newtab[ch+1]);  
                fprintf(out,"%02X", (int)newtab[ch]);  
                if(j != 7) {
                    fprintf(out,"%s", ",$");  
                }
                ch += 4;
            }
            fprintf(out,"\r\n");  
        }
        fprintf(out,"\r\n");  
    }
    fclose(out);
  	printf("File %s is created.\n\n", outfile);
    waitkey();
    return 0;
}


long filesize(char *fname)
{
    FILE *fp ;
    long size =0L;     

    fp = fopen(fname, "rb");
    fseek(fp, 0, SEEK_END); 
    size = ftell(fp);       
    fclose(fp); 
    return size;
}     


int byterev(int wert)
{
    int i, out;    
    char inbyte[9];
    char outbyte[9];

    for (i=0; i<8; i++) {
        inbyte[i] = '0';
        if(wert & (1 << (7 - i))) {
            inbyte[i] = '1';
        } 
    }
    inbyte[8] = '\0';
    strcpy(outbyte,inbyte);
    strrev(outbyte); 
    out = strtol( outbyte, NULL, 2);
    return out;
}


void waitkey()
{
	int k;
    printf("Press any Key to continue...");
	k = getch();
    printf("\n");
}


